#ifndef LIBMT_H
#define LIBMT_H
//
// LIBMT: This library defines several useful classes for use in multi
//  threaded programs
//  See libmt(3) for more information

#include "mt_common.h"
#include <sys/types.h>
#include <thread.h>
#include <synch.h>
#include <sys/socket.h>
#include <stdio.h>
#include <errno.h>

extern "C" {
 extern  char* sys_errlist[];
}
static void log_error(char* format,...) {
}

// TYPEDEFS

// thread_proc: function type passed to thr_create(3t)
typedef void* (*thread_proc)(void*);

/// THREAD CLASSES
// These classes provide an application framework for developing thread classes
// The methods on thread class fall into two categories: methods that can be 
// called by other methods to provide useful functionality, and methods that
// are overridden in subclasses to implement your application. 

class thread_class {
 public:
  // methods that should be over-ridden 
  virtual ~thread_class() {}
  virtual int options() { return THR_DETACHED;}
  virtual void before() {}
  virtual void after() {}
  virtual void panic() {delete this; thr_exit(0);}
  virtual void* run()=0;
  int set_priority(int level=0);
  // methods that can be called
  
  virtual void exit(void* result=0);
};

class daemon_thread : public thread_class {
 public:
  void before();
  void* run();
  void after();
  virtual int setup()=0;
  virtual void teardown()=0;
  virtual void* get_connection()=0;
  virtual thread_class* new_child(void* args)=0;
};
// exception classes

#ifdef HAVE_EXCEPTIONS
class thread_exit {
public:
  void* value;
  thread_exit(void* v =0) {
    value = v;
  }
};
#endif

// START_THREAD: This function creates and starts a thread for the 
// thread_class instance C. 

thread_t start_thread(thread_class* c);


// The classes provide several useful facilities for use in multi-
// threaded applications

class shared_object {
protected:
  mutex_t object_lock;
public:
  shared_object() { mutex_init(&object_lock,USYNC_THREAD,0); }
  int lock() { return mutex_lock(&object_lock); }
  int unlock() { return mutex_unlock(&object_lock); }
};

class shared_condition : public shared_object {
  cond_t the_condition;
public:

  shared_condition() : shared_object() {
    cond_init(&the_condition,USYNC_THREAD,0);
  }
  virtual  ~shared_condition()  {
    cond_destroy(&the_condition);
  }
  virtual int test() {return 1;}
  int signal();
  int broadcast(); 
  int wait(); 
};
  
  
class shared_counter :public shared_object {
  int value;
public:
  shared_counter(int val=0) : shared_object() {
    lock();  value = val; unlock();
  }
  int get_value() {
    int tmp;
    lock();  tmp = value;  unlock();
    return tmp;
  }
  int inc(int plus=1) {
    int tmp;
    lock();  value += plus;  tmp = value;  unlock();
    return tmp;
  }
  int dec(int minus=1) {
    int tmp;
    lock() ; value -= minus; tmp = value ; unlock();
    return tmp;
  }
  int operator++() {
    return inc(1);
  }
  int operator--() {
    return dec(1);
  }
   operator int() {
     return get_value();
   }
};

  
#endif

